@@ -48,22 +48,23 @@ properties_apply_callback(gpointer data)
 						    "pixmap");
 	GtkWidget    *tipentry = g_object_get_data (G_OBJECT (drawer->properties),
 						    "tooltip");
-	char         *s;
 	const char   *cs;
 
 	g_free (drawer->pixmap);
 	drawer->pixmap = NULL;
 	g_free (drawer->tooltip);
 	drawer->tooltip = NULL;
+
 	drawer->pixmap = gnome_icon_entry_get_filename (GNOME_ICON_ENTRY (pixentry));
 	if (string_empty (drawer->pixmap) ||
 	    access (drawer->pixmap, R_OK) != 0) {
 		drawer->pixmap = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
 							    "panel-drawer.png", TRUE, NULL);
 	}
+
 	button_widget_set_pixmap (BUTTON_WIDGET (drawer->button),
 				  drawer->pixmap, -1);
-	g_free(s);
+
 	cs = gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(tipentry))));
 	if (string_empty (cs))
 		drawer->tooltip = NULL;
@@ -322,32 +323,32 @@ key_press_drawer (GtkWidget *widget, GdkEventKey *event, gpointer data)
 		switch (event->keyval) {
 		case GDK_Up:
 		case GDK_KP_Up:
-			if (parent->orient == GTK_ORIENTATION_HORIZONTAL)
-				dir = GTK_DIR_TAB_BACKWARD;
-			else
-				return TRUE;
-			break;
 		case GDK_Left:
 		case GDK_KP_Left:
-			if (parent->orient == GTK_ORIENTATION_VERTICAL)
+			switch (BASEP_WIDGET (drawerw)->state) {
+			case BASEP_SHOWN:
+			case BASEP_AUTO_HIDDEN:
 				dir = GTK_DIR_TAB_BACKWARD;
-			else
-				return TRUE;
+				break;
+			default:
+                                return TRUE;
+				break;
+			}
 			break;
 		case GDK_Down:
 		case GDK_KP_Down:
-			if (parent->orient == GTK_ORIENTATION_HORIZONTAL)
-				dir = GTK_DIR_TAB_FORWARD;
-			else
-				return TRUE;
-			break;
 		case GDK_Right:
 		case GDK_KP_Right:
-			if (parent->orient == GTK_ORIENTATION_VERTICAL)
+			switch (BASEP_WIDGET (drawerw)->state) {
+			case BASEP_SHOWN:
+			case BASEP_AUTO_HIDDEN:
 				dir = GTK_DIR_TAB_FORWARD;
-			else
-				return TRUE;
-			break;
+				break;
+			default:
+                                return TRUE;
+				break;
+			}
+                        break;
 		case GDK_Escape:
 			switch (BASEP_WIDGET (drawerw)->state) {
 			case BASEP_SHOWN:
@@ -374,6 +375,48 @@ key_press_drawer (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	}
 }
 
+/*
+ * This function implements Esc moving focus from the drawer to the drawer
+ * icon and closing the drawer and Shift+Esc moving focus from the drawer
+ * to the drawer icon without closing the drawer when focus is in the drawer.
+ */
+static gboolean
+key_press_drawer_widget (GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+	Drawer *drawer;
+	DrawerWidget *drawerw;
+	PanelWidget *parent;
+	gboolean ret_val = FALSE;
+
+	switch (event->keyval) {
+	case GDK_Escape: {
+		drawer = data;
+		parent = PANEL_WIDGET (drawer->button->parent);
+		drawerw = DRAWER_WIDGET (drawer->drawer);
+
+		if (event->state == GDK_SHIFT_MASK) {
+			gtk_window_present (GTK_WINDOW (parent->panel_parent));
+			ret_val = TRUE;
+		} else if (event->state == 0) {
+			gtk_window_present (GTK_WINDOW (parent->panel_parent));
+			switch (BASEP_WIDGET (drawerw)->state) {
+			case BASEP_SHOWN:
+			case BASEP_AUTO_HIDDEN:
+				drawer_widget_close_drawer (drawerw, parent->panel_parent);
+				break;
+			default:
+				break;
+			}
+			ret_val = TRUE;
+			break;
+		} 
+	}
+	default:
+		break;
+	}
+	return ret_val;
+}
+
 static void  
 drag_data_get_cb (GtkWidget          *widget,
 		  GdkDragContext     *context,
@@ -475,6 +518,8 @@ create_drawer_applet (GtkWidget   *drawer_panel,
 			    G_CALLBACK (focus_out_drawer), drawer);
 	g_signal_connect (G_OBJECT (drawer->button), "key_press_event",
 			    G_CALLBACK (key_press_drawer), drawer);
+	g_signal_connect (G_OBJECT (drawer->drawer), "key_press_event",
+			    G_CALLBACK (key_press_drawer_widget), drawer);
 
 	g_object_set_data (G_OBJECT (drawer_panel), DRAWER_PANEL_KEY, drawer);
 	gtk_widget_queue_resize (GTK_WIDGET (drawer_panel));
@@ -620,10 +665,10 @@ load_drawer_applet (gchar       *mypanel_id,
 		panel_applet_add_callback (drawer->info,
 					   "properties",
 					   GTK_STOCK_PROPERTIES,
-					   _("Properties"));
+					   _("_Properties"));
 
 	panel_applet_add_callback (
-		drawer->info, "help", GTK_STOCK_HELP, _("Help"));
+		drawer->info, "help", GTK_STOCK_HELP, _("_Help"));
 
 	return drawer;
 }
